@@ -8,17 +8,10 @@ from unittest import mock
 import pytest
 from moto import mock_cloudformation, mock_ec2
 
+from remote_docker_aws.config import RemoteDockerConfigProfile
 from remote_docker_aws.core import (
-    create_keypair,
-    create_instance,
-    delete_instance,
+    create_remote_docker_client,
     get_ec2_client,
-    get_instance_state,
-    get_ip,
-    start_tunnel,
-    start_instance,
-    stop_instance,
-    sync,
 )
 
 
@@ -28,15 +21,22 @@ REGION = "ca-central-1"
 
 patch_exec = mock.patch("os.execvp", autospec=True)
 patch_run = mock.patch("subprocess.run", autospec=True)
-patch_get_ip = mock.patch("remote_docker_aws.core.get_ip", autospec=True)
+patch_get_ip = mock.patch(
+    "remote_docker_aws.core.RemoteDockerClient.get_ip", autospec=True
+)
 
 
 def generate_ssh_public_key():
     key = rsa.generate_private_key(
         backend=crypto_default_backend(), public_exponent=65537, key_size=2048
     )
-    return key.public_key().public_bytes(
-        crypto_serialization.Encoding.OpenSSH, crypto_serialization.PublicFormat.OpenSSH
+    return (
+        key.public_key()
+        .public_bytes(
+            crypto_serialization.Encoding.OpenSSH,
+            crypto_serialization.PublicFormat.OpenSSH,
+        )
+        .decode()
     )
 
 
@@ -67,9 +67,26 @@ def fix_dependency_conflict():
 @mock_cloudformation
 @mock_ec2
 class TestCore:
-    def test_get_ip_when_no_instances_running(self):
+    @pytest.fixture
+    def remote_docker_client(self):
+        config = RemoteDockerConfigProfile(
+            config_dict=dict(
+                project_code="mock_project",
+                aws_region=REGION,
+                instance_service_name="mock_instance",
+                instance_type="c4.xlarge",
+                local_port_forwards=dict(test_local={"80": "80"}),
+                remote_port_forwards=dict(test_remote={"8080": "8080"}),
+                key_path=KEY_PATH,
+                watched_directories=["/fake/dir"],
+                sync_ignore_patterns_git=["test.py"],
+            )
+        )
+        return create_remote_docker_client(config)
+
+    def test_get_ip_when_no_instances_running(self, remote_docker_client):
         with pytest.raises(RuntimeError) as exc:
-            get_ip(aws_region=REGION)
+            remote_docker_client.get_ip()
             assert (
                 str(exc.value)
                 == "There are no valid reservations, did you create the instance?"
@@ -77,37 +94,32 @@ class TestCore:
 
     @patch_exec
     @mock.patch("remote_docker_aws.core.wait_until_port_is_open", autospec=True)
-    def test_creates_and_interacts_with_instance(self, mock_wait, mock_execvp):
-        create_instance(
-            ssh_key_path="mock_key_path", aws_region=REGION, instance_type="c4.xlarge"
-        )
+    def test_creates_and_interacts_with_instance(
+        self, mock_wait, mock_execvp, remote_docker_client
+    ):
+        remote_docker_client.create_instance()
         mock_wait.assert_called_once()
         mock_execvp.assert_called_once()
 
-        assert is_valid_ip(get_ip(REGION))
-        assert get_instance_state(REGION) == "running"
+        assert is_valid_ip(remote_docker_client.get_ip())
+        assert remote_docker_client.get_instance_state() == "running"
 
-        stop_instance(REGION)
-        assert get_instance_state(REGION) == "stopped"
+        remote_docker_client.stop_instance()
+        assert remote_docker_client.get_instance_state() == "stopped"
 
-        start_instance(REGION)
-        assert get_instance_state(REGION) == "running"
+        remote_docker_client.start_instance()
+        assert remote_docker_client.get_instance_state() == "running"
 
-        delete_instance(REGION)
+        remote_docker_client.delete_instance()
         with pytest.raises(RuntimeError):
-            get_ip(aws_region=REGION)
+            remote_docker_client.get_ip()
 
     @patch_get_ip
     @patch_exec
     @patch_run
-    def test_sync(self, mock_run, mock_execvp, mock_get_ip):
+    def test_sync(self, mock_run, mock_execvp, mock_get_ip, remote_docker_client):
         mock_get_ip.return_value = "1.2.3.4"
-        sync(
-            dirs=["/fake/dir"],
-            ssh_key_path="/fake_key_path",
-            sync_ignore_patterns_git=["test.py"],
-            aws_region=REGION,
-        )
+        remote_docker_client.sync()
 
         assert mock_run.call_count == 2
         call_1, call_2 = mock_run.call_args_list
@@ -155,17 +167,9 @@ class TestCore:
 
     @patch_get_ip
     @patch_run
-    def test_tunnel(self, mock_run, mock_get_ip):
+    def test_tunnel(self, mock_run, mock_get_ip, remote_docker_client):
         mock_get_ip.return_value = "1.2.3.4"
-        local_forwards = dict(test_local={"80": "80"})
-        remote_forwards = dict(test_remote={"8080": "8080"})
-
-        start_tunnel(
-            ssh_key_path="/fake_key_path",
-            local_forwards=local_forwards,
-            remote_forwards=remote_forwards,
-            aws_region=REGION,
-        )
+        remote_docker_client.start_tunnel()
 
         mock_run.assert_called_once()
         assert mock_run.call_args[0][0] == [
@@ -190,12 +194,10 @@ class TestCore:
         ]
 
     @patch_run
-    def test_create_keypair(self, mock_run):
+    def test_create_keypair(self, mock_run, remote_docker_client):
         with mock.patch("builtins.open") as mock_open:
-            mock_open.side_effect = mock.mock_open(
-                read_data=generate_ssh_public_key().decode()
-            )
-            create_keypair(KEY_PATH, REGION)
+            mock_open.side_effect = mock.mock_open(read_data=generate_ssh_public_key())
+            remote_docker_client.create_keypair()
 
         assert mock_run.call_count == 2
         call_1, call_2 = mock_run.call_args_list
